package de.hdm_stuttgart.mi.sd1.aufgabe2;

/**
 * Darstellungen von Breitenangaben entweder als (Grad, Minuten, Sekunden) oder dezimal.
 *
 * <p>Längen- oder Breitenangaben können als (Grad, Winkelminuten, Winkelsekunden) dargestellt werden. Dabei gilt:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Angaben in Grad, Minute und Sekunde</caption>
 *     <tr>
 *         <td style="width: 35em">
 *           <ul>
 *             <li>360° Grad entsprechen einem Vollkreis.</li>
 *             <li>60 Winkelminuten ergeben 1 Grad: 60´ == 1°.</li>
 *             <li>60 Winkelsekunden ergeben eine Winkelminute: 60" = 1´.</li>
 *           </ul>
 *         </td>
 *         <td style="width: 30em"><b>Beispiel:</b> Die geographische Breitenangabe von Stuttgart lautet
 *         (<b style="color: red;">48°</b>,<b style="color: green;">46′</b>,<b style="color: blue;">56″</b>). Die Angabe
 *         »Nord« wird in dieser Aufgabe ignoriert.</td>
 *     </tr>
 * </table>
 *
 * <p>Alternativ gibt es auch eine <em>dezimale</em> Graddarstellung: Diese geht ebenfalls von einem Vollkreis zu 360°
 * aus. Allerdings werden die Minuten- und Sekundenanteile als Dezimalbruch angegeben. Die geographische Breite
 * (<b style="color: red;">48°</b>, <b style="color: green;">46′</b>, <b style="color: blue;">56″</b>) beispielsweise
 * von Stuttgart ergibt sich in dieser Darstellung zu:</p>
 *
 * \[\color{red}{48} + {\color{green}{46}\over 60} + {\color{blue}{56} \over {60 \times 60}} = 48.782222\dots \]
 *
 * <p>Instanzen von {@link Geo} können aus beiden Darstellungen erzeugt werden:</p>
 *
 * <pre>final Geo stuttgartBreite1 = new Geo(<b style="color: red;">48</b>, <b style="color: green;">46</b>,<b style="color: blue;">56</b>), // Grad, Minuten, Sekunden
 *          stuttgartBreite2 = new Geo(48.782222); // Dezimalangabe: Identische Breite bis auf Rundungsfehler</pre>
 *
 * <section class="implementationHints">
 *     <h2>Hinweis:</h2>
 *
 *     <p>Verwenden Sie als Interndarstellung die Angabe in (Grad, Minuten, Sekunden). Sie müssen dann
 *     für den Konstruktor aus einer dezimalen <code>double</code> Gradangabe im ersten Schritt Werte wie 48.782222 in
 *     den ganzzahligen Gradanteil <b style="color: red;">48°</b> und den gebrochenen Anteil 0.782222 zerlegen. Dazu
 *     kann die Methode {@link Math#floor(double)} in Zusammenarbeit mit einer cast Operation verwendet werden.</p>
 * </section>
 *
 * <p>Jede Instanz von {@link Geo} stellt beide Darstellungen bereit:</p>
 *
 * <pre><code class="java"> final Geo breite = ... ;
 *
 * final int grad = breite.grad,
 *        minuten = breite.minuten,
 *       sekunden = breite.sekunden;
 *
 * final double breiteDezimal = breite.getDezimal();</code></pre>
 *
 * <p>Instanzen von {@link Geo} werden im Standard wie folgt ausgegeben:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Koordinatendarstellung als (gg°, mm', ss")</caption>
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo stuttgartBreite = new Geo(48.782222);
 * System.out.println(stuttgartBreite);</code></pre>
 *         </td>
 *         <td>(48°,46´,56´´)</td>
 *     </tr>
 * </table>
 *
 * <p>Das Format der Ausgabe kann zur Laufzeit global für alle Instanzen zwischen (gg°, mm', ss") und Dezimal
 *    gewechselt werden. Für die Dezimaldarstellung werden sechs Nachkommastellen ausgegeben:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Wechsel der Darstellung</caption>
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo stuttgartBreite = new Geo(48.782222);
 * Geo.nutzeDezimalDarstellung(true);
 * System.out.println(stuttgartBreite);</code></pre>
 *         </td>
 *         <td>48.782222</td>
 *     </tr>
 * </table>
 *
 * <section class="implementationHints">
 *     <h3>Hinweis:</h3>
 *
 *     <p>Nutzen Sie {@link java.text.DecimalFormat} und verwenden Sie
 *     {@link java.text.DecimalFormat#setMaximumIntegerDigits(int)} sowie
 *     {@link java.text.DecimalFormat#setMaximumFractionDigits(int)}.</p>
 * </section>
 *
 * <p>Instanzen von Geo sollen auf Wertgleichheit geprüft werden können:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Wertgleichheit von {@link Geo} Instanzen</caption>
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo
 *   g1 =      new Geo(33, 53, 19),
 *   g2 =      new Geo(45, 22, 13),
 *   g1_klon = new Geo(33, 53, 19);
 *
 * System.out.println(g1.equals(g2));
 * System.out.println(g1.equals(g1_klon));</code></pre>
 *         </td>
 *         <td>
 *             <pre> false
 * true</pre>
 *         </td>
 *     </tr>
 * </table>
 *
 * <p>Winkelangaben sollen aufsteigend sortierbar sein:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Sortierung von {@link Geo} instanzen</caption>
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo
 * stuttgartBreite = new Geo(48, 46, 56),
 * beirutBreite = new Geo(33, 53, 19),
 * atlantaBreite = new Geo(33, 44, 56);
 *
 * final Geo[] breitenAngaben =
 *    {stuttgartBreite, beirutBreite, atlantaBreite};
 *
 * Arrays.sort(breitenAngaben);
 *
 * System.out.println("Breitengrade Stuttgart, Beirut und Atlanta\n"
 *     + "aufsteigend sortiert:");
 *
 * for (final Geo g: breitenAngaben) {
 *   System.out.println(g);
 * }</code></pre>
 *         </td>
 *         <td>
 *             <pre> Breitengrade Stuttgart, Beirut und Atlanta
 * aufsteigend sortiert:
 * (33°,44´,56´´)
 * (33°,53´,19´´)
 * (48°,46´,56´´)</pre>
 *         </td>
 *     </tr>
 * </table>
 *
 * <section class="implementationHints">
 *     <h3>Hinweis:</h3>
 *
 *     <p>Beachten Sie {@link java.lang.Comparable}.</p>
 *
 * </section>
 *
 * <p>Instanzen von {@link Geo} sind unveränderbar (immutable). Der folgende Code führt beispielsweise zu einem Compile
 * time Fehler:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Immutable instance attributes</caption>
 *     <tr>
 *         <th>Code</th>
 *         <th>Fehlermeldung Compiler</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo g = new Geo(22);
 * g.sekunden = 44;</code></pre>
 *         </td>
 *         <td style="color: red;">Cannot assign a value to final variable 'sekunden'</td>
 *     </tr>
 * </table>
 *
 * <section class="implementationHints">
 *     <h3>Hinweis:</h3>
 *
 *     <p>Die Klasse {@link GeoUsageSample} enthält auskommentierten Code, welcher Ihnen als Leitlinie für die
 *       Verwendung von {@link Geo} dienen kann.</p>
 *
 * </section>
 *
 */
public class Geo {
  // TODO: Implementiere mich!
}
